#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['zazo']

package_data = \
{'': ['*']}

install_requires = \
['packaging', 'six']

setup(name='zazo',
      version='0.0.0a2',
      description='An Extensible Dependency Resolver written in Python',
      author='Pradyun Gedam',
      author_email='pradyunsg@gmail.com',
      url='https://github.com/pradyunsg/zazo',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
     )
